@@ -126,6 +126,7 @@ polkit_gnome_authenticator_class_init (PolkitGnomeAuthenticatorClass *klass)
    * PolkitGnomeAuthenticator::completed:
    * @authenticator: A #PolkitGnomeAuthenticator.
    * @gained_authorization: Whether the authorization was gained.
+   * @dismissed: Whether the dialog was dismissed.
    *
    * Emitted when the authentication is completed. The user is supposed to dispose of @authenticator
    * upon receiving this signal.
@@ -136,9 +137,10 @@ polkit_gnome_authenticator_class_init (PolkitGnomeAuthenticatorClass *klass)
                                             0,                      /* class offset     */
                                             NULL,                   /* accumulator      */
                                             NULL,                   /* accumulator data */
-                                            g_cclosure_marshal_VOID__BOOLEAN,
+                                            NULL, /* use generic marshaller */
                                             G_TYPE_NONE,
-                                            1,
+                                            2,
+                                            G_TYPE_BOOLEAN,
                                             G_TYPE_BOOLEAN);
 }
 
@@ -209,10 +211,19 @@ polkit_gnome_authenticator_new (const gchar     *action_id,
   PolkitGnomeAuthenticator *authenticator;
   GList *l;
   guint n;
+  GError *error;
 
   authenticator = POLKIT_GNOME_AUTHENTICATOR (g_object_new (POLKIT_GNOME_TYPE_AUTHENTICATOR, NULL));
 
-  authenticator->authority = polkit_authority_get ();
+  error = NULL;
+  authenticator->authority = polkit_authority_get_sync (NULL /* GCancellable* */, &error);
+  if (authenticator->authority == NULL)
+    {
+      g_critical ("Error getting authority: %s", error->message);
+      g_error_free (error);
+      goto error;
+    }
+
   authenticator->action_id = g_strdup (action_id);
   authenticator->message = g_strdup (message);
   authenticator->icon_name = g_strdup (icon_name);
@@ -462,7 +473,10 @@ do_initiate (gpointer user_data)
     }
 
  out:
-  g_signal_emit_by_name (authenticator, "completed", authenticator->gained_authorization);
+  g_signal_emit_by_name (authenticator,
+                         "completed",
+                         authenticator->gained_authorization,
+                         authenticator->was_cancelled);
 
   g_object_unref (authenticator);
 
